/* Writes out the actual Javadoc files.

Copyright (c) 2003-2004 The Regents of the University of California.
All rights reserved.
Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

PT_COPYRIGHT_VERSION_2
COPYRIGHTENDKEY

*/

package doc.doclets;

import com.sun.tools.doclets.ClassTree;
import com.sun.tools.doclets.DocletAbortException;
import com.sun.tools.doclets.VisibleMemberMap;
import com.sun.tools.doclets.standard.ClassWriter;
import com.sun.tools.doclets.standard.ConfigurationStandard;
import com.sun.tools.doclets.standard.HtmlStandardWriter;
import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.PackageDoc;
import com.sun.javadoc.ProgramElementDoc;
import com.sun.javadoc.Tag;
import java.io.IOException;

import java.lang.reflect.InvocationTargetException;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.sdm.spa.util.ActorInspector;
import doc.taglets.DescriptionTaglet;

import ptolemy.kernel.util.KernelException;


//////////////////////////////////////////////////////////////////////////
////ActorClassWriter
/**
* Writes out the actual Javadoc files.
*
* @author Ilkay Altintas
* @author xiaowen
* @version $Id: ActorClassWriter.java,v 1.1 2005/03/03 17:32:17 altintas Exp $
* @category.name documentation
* @category.name javadoc
*/
public class ActorClassWriter extends ClassWriter {


    /**
     * @throws IOException
     * @throws DocletAbortException
     */
    public ActorClassWriter(ConfigurationStandard configuration,
                       String path, String filename, ClassDoc classdoc,
                       ClassDoc prev, ClassDoc next, ClassTree classtree,
                       boolean generateFieldLinks)
        throws IOException {

        super(configuration, path, filename, classdoc, prev, next, classtree);

        PackageDoc pkg = classdoc.containingPackage();
        String pkgname = (pkg != null)? pkg.name(): "";
        String clname = classdoc.name();

        ActorInspector ai = null;

        try {
            if (pkgname.length() > 0) {
               ai = new ActorInspector(pkgname + "." + clname);
            } else {
               ai = new ActorInspector(clname);
            }
        } catch (Exception ex){
            System.out.println("Exception when generating actor instance for "
            + pkgname + "." + clname + ":" + ex);
            ai = null;
        }

        _actorInspector = ai;
        _generateFieldLinks = generateFieldLinks;
    }

    public ActorClassWriter(
        ConfigurationStandard configuration,
        String path, String filename, ClassDoc classdoc,
        ClassDoc prev, ClassDoc next, ClassTree classtree)
        throws IOException {

        this(configuration, path, filename, classdoc, prev, next, classtree, true);
    }

    /**
     * @return Whether the class is an actor.
     */
    public boolean isActor() {
        return null != _actorInspector;
    }

    /**
     * Generate the class file contents.
     */
    public void generateClassHeader() {
        String cltype = getText(classdoc.isInterface()?
                                    "doclet.Interface":
                                    "doclet.Class");
        PackageDoc pkg = classdoc.containingPackage();
        String pkgname = (pkg != null)? pkg.name(): "";
        String clname = classdoc.name();
        String label = cltype + " " + clname;

        printHtmlHeader(clname,
            configuration.metakeywords.getMetaKeywords(classdoc));
        navLinks(true);
        hr();
        println("<!-- ======== START OF CLASS DATA ======== -->");
        //h3();
        //if (pkgname.length() > 0) {
        //    font("-1"); print(pkgname); fontEnd(); br();
        //}
        //print(label);
        //print(cltype);
        //h3End();
        h2();
        print("<FONT COLOR=\"#660000\">" + clname + "</FONT>");
        h2End();
    }


    /**
     * @exception ClassNotFoundException If thrown by ActorInspector.
     * @exception ClassCastException If thrown by ActorInspector.
     */
    public void generateClassActorInfo()
        throws ClassNotFoundException, ClassCastException,
               InstantiationException, IllegalAccessException,
               InvocationTargetException, KernelException,
               NoSuchMethodException {

        if(null == _actorInspector) {
            return;
        }

        String clname = classdoc.name();

        println("<!-- ======== START OF THE ACTOR DATA ======== -->");

        h4();
        print("<FONT COLOR=\"#000066\">THE USAGE OF THE ACTOR:</FONT>");
        h4End();

        if(!configuration.nocomment) {
            // generate documentation for the class.
            if (classdoc.inlineTags().length > 0) {
                printInlineComment(classdoc);
            }
        }

        h4();
        print("<FONT COLOR=\"#000066\">Description of Ports and Parameters:</FONT>");
        h4End();

        List inports = _actorInspector.getInputPorts();
        List outports = _actorInspector.getOutputPorts();
        List attrs = _actorInspector.getAttributeList();

        table(0,3,3);
        tr();td();tdEnd();
        td();print("<FONT COLOR=\"red\">NAME</FONT>");tdEnd();
        td();print("<FONT COLOR=\"red\">TYPE</FONT>");tdEnd();
        td();print("<FONT COLOR=\"red\">IS MULTIPORT?</FONT>");tdEnd();
        td();
        //tdWidth("100");
        print("<FONT COLOR=\"red\">EXPLANATION</FONT>");tdEnd();
        trEnd();//br();
        tr();td();bold("<FONT COLOR=\"red\">Input ports: </FONT>");tdEnd();trEnd();

        DescriptionTaglet dt = new DescriptionTaglet();
        Tag[] tags = classdoc.tags(dt.getName());
        HashMap mapNameToDesc = new HashMap();
        for(int i = 0; i < tags.length; i++) {
            Tag tag = tags[i];
            String[] pair = dt.getTagString(tag);
            mapNameToDesc.put(pair[0], pair[1]);
        }

        HashMap mapNameToFieldDesc = new HashMap();
        VisibleMemberMap vmm = new VisibleMemberMap(classdoc, VisibleMemberMap.FIELDS, true);
        List list = vmm.getMembersFor(classdoc);
        Iterator it = list.iterator();
        while(it.hasNext()) {
            ProgramElementDoc ped = (ProgramElementDoc) it.next();
            String commentText = ped.commentText().trim();
            if(!commentText.equals("")) {
                mapNameToFieldDesc.put(ped.name(), commentText);
            }
            tags = ped.tags(dt.getName());
            for(int i = 0; i < tags.length; i++) {
                Tag tag = tags[i];
                mapNameToFieldDesc.put(ped.name(), tag.text());
            }
        }

        Iterator inputs = inports.iterator();
        while (inputs.hasNext()) {
           ActorInspector.PortInfo port = (ActorInspector.PortInfo) inputs.next();
           // get properties
           tr();
           td();tdEnd();
           td();bold(port.name);tdEnd();
           td();print("<I>" + port.type.toString() + "</I>");tdEnd();
           td();print(port.isMultiport);tdEnd();
           //../../../org/sdm/spa/

            td();
            //tdWidth("100");
            if(mapNameToDesc.containsKey(port.name)) {
                print((String) mapNameToDesc.get(port.name));
            } else if(mapNameToFieldDesc.containsKey(port.name)) {
                print((String) mapNameToFieldDesc.get(port.name));
                writeFieldLink(clname + ".html#" + port.name);
            } else {
                print("No description available.");
            }
            tdEnd();
            trEnd();
        }
        //br();
        tr();td();bold("<FONT COLOR=\"red\">Output ports: </FONT>");tdEnd();trEnd();
        Iterator outputs = outports.iterator();
        while (outputs.hasNext()) {
           ActorInspector.PortInfo port = (ActorInspector.PortInfo) outputs.next();
           tr();
           td();tdEnd();
           td();bold(port.name);tdEnd();
           td();print("<I>" + port.type.toString() + "</I>");tdEnd();
           td();print(port.isMultiport);tdEnd();

            td();
            //tdWidth("100");
            if(mapNameToDesc.containsKey(port.name)) {
                print((String) mapNameToDesc.get(port.name));
            } else if(mapNameToFieldDesc.containsKey(port.name)) {
                print((String) mapNameToFieldDesc.get(port.name));
                writeFieldLink(clname + ".html#" + port.name);
            } else {
                print("No description available.");
            }
            tdEnd();
            trEnd();
        }
        //br();
        tr();td();bold("<FONT COLOR=\"red\">Parameters: </FONT>");tdEnd();trEnd();
        Iterator attrsIter = attrs.iterator();
        while (attrsIter.hasNext()) {
           ActorInspector.AttributeInfo at = (ActorInspector.AttributeInfo) attrsIter.next();
            if(at.name.startsWith("_")) {
                continue;
            }
           tr();
           td();tdEnd();
           td();bold(at.name);tdEnd();
           td();print("N/A");tdEnd();
           td();print("N/A");tdEnd();

            td();
            //tdWidth("100");
            if(mapNameToDesc.containsKey(at.name)) {
                print((String) mapNameToDesc.get(at.name));
            } else if(mapNameToFieldDesc.containsKey(at.name)) {
                print((String) mapNameToFieldDesc.get(at.name));
                writeFieldLink(clname + ".html#" + at.name);
            } else {
                print("No description available.");
            }
            tdEnd();
            trEnd();
        }
        tableEnd();
    }

    public void generateClassFooter() {
        hr();

        h4();
        print("<FONT COLOR=\"#000066\">CLASS DOCUMENTATION:</FONT>");
        h4End();

        // if this is a class (not an interface) then generate
        // the super class tree.
        if (classdoc.isClass()) {
            pre();
            printTreeForClass(classdoc);
            preEnd();
        }

        printSuperImplementedInterfacesInfo();

        printSubClassInterfaceInfo();

        if (classdoc.isInterface()) {
            printImplementingClasses();
        }

        printEnclosingInfo();

        hr();

        printDeprecated();

        printClassDescription();
        p();
        if(!configuration.nocomment) {
            if (null != _actorInspector) {
            	// generate documentation for the class.
        /*    if (classdoc.inlineTags().length > 0) {
                printInlineComment(classdoc);
                p();
            }
          */  // Print Information about all the tags here
            generateTagInfo(classdoc);
            hr();
            p();
            }
            else {
            	// generate documentation for the class.
                if (classdoc.inlineTags().length > 0) {
                        printInlineComment(classdoc);
                        p();
                    }
                  // Print Information about all the tags here
                    generateTagInfo(classdoc);
                    hr();
                    p();  	
            }
        } else {
            hr();
        }

        printAllMembers();

        println("<!-- ========= END OF CLASS DATA ========= -->");
        hr();
        navLinks(false);
        printBottom();
        printBodyHtmlEnd();
    }

    public void generateClassFinish() {

        PackageDoc cp = classdoc.containingPackage();

        if((configuration.packages == null ||
                Arrays.binarySearch(configuration.packages, cp) < 0) &&
            ! containingPackagesSeen.contains(cp.name())){

            //Only copy doc files dir if the containing package is not documented
            //AND if we have not documented a class from the same package already.
            //Otherwise, we are making duplicate copies.
            HtmlStandardWriter.copyDocFiles(configuration, getSourcePath(configuration, classdoc.containingPackage()),
                        path + fileseparator + DOC_FILES_DIR_NAME, true);
            containingPackagesSeen.add(cp.name());
        }
    }

    private void writeFieldLink(String link) {
        if(_generateFieldLinks) {
            print(" <a href=\"" + link + "\">More ...</a>");
        }
    }

    private final ActorInspector _actorInspector;
    private final boolean _generateFieldLinks;
}

