/* The doclet that adds a manual file like description
in front of the standard javadoc.

Copyright (c) 2003-2004 The Regents of the University of California.
All rights reserved.
Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

PT_COPYRIGHT_VERSION_2
COPYRIGHTENDKEY

*/

package doc.doclets;

import java.io.IOException;
import java.util.Arrays;

import com.sun.javadoc.ClassDoc;
import com.sun.javadoc.RootDoc;
import com.sun.tools.doclets.ClassTree;
import com.sun.tools.doclets.DirectoryManager;
import com.sun.tools.doclets.DocletAbortException;
import com.sun.tools.doclets.standard.Standard;

//////////////////////////////////////////////////////////////////////////
//// ExtendedActorDoclet
/**
 * Given ...
 *
 * @author Ilkay Altintas
 * @version $Id: ExtendedActorDoclet.java,v 1.2 2005/03/03 17:32:17 altintas Exp $
 * @category.name documentation
 * @category.name javadoc
 */
public class ExtendedActorDoclet extends Standard{

	public ExtendedActorDoclet () {
		
	}
	
    /***************************************************
     *  Instance methods
     ***************************************************/

    /**
     * Extends the standard doclets startGeneration method to add a logging message.
     */
    protected void startGeneration(RootDoc root) {
        if(!configuration.quiet) {
            System.out.println("\nExtendedActorDoclet (jdk 1.4.2)");
        }
        super.startGeneration(root);
    }

    /********************    Public methods  *********************/

    /**
     * The =public static void main= of doclets.
     */
    public static boolean start(RootDoc root) throws IOException {
        ExtendedActorDoclet myDoclet;

        try {
            (myDoclet = new ExtendedActorDoclet()).startGeneration(root);
        } catch (DocletAbortException exc) {
            return false;
        }

       // myDoclet.configuration.tagletManager.printReport();
        return true;
    }

    /**
     * Instantiate ClassWriter for each Class within the ClassDoc[]
     * passed to it and generate Documentation for that.
     */
    protected void generateClassCycle(ClassDoc[] arr, ClassTree classtree) {
        Arrays.sort(arr);
        for(int i = 0; i < arr.length; i++) {
            if (configuration.nodeprecated &&
                arr[i].tags("deprecated").length > 0) {
                continue;
            }
            ClassDoc prev = (i == 0)?
                null:
                arr[i-1];
            ClassDoc curr = arr[i];
            ClassDoc next = (i+1 == arr.length)?
                null:
                arr[i+1];

            try {
                String pkgpath =
                    DirectoryManager.getDirectoryPath(curr.containingPackage());
                String filename = curr.name() + ".html";

                ActorClassWriter clsgen = new ActorClassWriter(configuration,
                        pkgpath, filename, curr,
                        prev, next, classtree);

                clsgen.generateClassHeader();

                // Output actor information.
                // This may throw exceptions if the input isn't an actor.
                try {
                    clsgen.generateClassActorInfo();
                } catch(ClassCastException e) {
                    // Ignore.
                } catch(ClassNotFoundException e) {
                    // Ignore.
                } catch(Exception e) {
                    // Unexpected error.
                    e.printStackTrace();
                }

                clsgen.generateClassFooter();
                clsgen.generateClassFinish();

                clsgen.close();
            } catch (IOException exc) {
                configuration.standardmessage.error(
                    "doclet.exception_encountered",
                    exc.toString(), curr.name());
            }
        }
    }
}
