#ifndef __GIOPTR_H__

#define __GIOPTR_H__



#include "gioapi.h"



#ifdef __GARPWIN

#include <windows.h>

#else



typedef int HMODULE;



#define FreeLibrary( hmodule )  true

#define GetProcessAddress( hmodule, sModName )  NULL



#endif



// ==================================================================

//  Function Pointer Types

// ==================================================================



typedef int (*TFPGridIOSetup)();

typedef int (*TFPGridIOExit)();

typedef int (*TFPCellLyrExists)(char *);

typedef int (*TFPGridExists)(char *grid_name);

typedef int (*TFPGridCopy)(char *from_grid, char *to_grid);

typedef int (*TFPGridRename)(char *from_grid, char *to_grid);

typedef int (*TFPGridDelete)(char *grid_name);

typedef int (*TFPGridKill)(char *grid_name, int option);

typedef int (*TFPDescribeGridDbl)(char *grdnam, double *cellsz, int *gridsz,

                  double *box, double *sta,

                  int *datatyp, int *nclass, int *reclen);

typedef int (*TFPBndCellRead)(char *grdnam, double *box);

typedef int (*TFPCellLyrExists)(char *clyr_name);

typedef int (*TFPCellLayerCreate )(char *clyr_name, int rdwrflag, int iomode,

                         int celltype, double cellsize, double box[4]);

typedef int (*TFPCellLayerCreateTiled )(char *clyr_name, int rdwrflag, int iomode,

                          int  celltype, double cellsize, double box[4],

                      int blockxcells, int blockycells);

typedef int (*TFPCellLayerOpen )(char *clyr_name, int rdwrflag, int iomode,

                   int  *celltype, double *cellsize);

typedef int (*TFPCellLyrClose )(int channel);

typedef int (*TFPCellLyrCloseNoVat )(int channel);

typedef int (*TFPCellLyrDelete )(char *clyr_name);

typedef int (*TFPCellLyrGetIO)(int channel);

typedef int (*TFPCellLyrSetIO)(int channel, int mode);

typedef int (*TFPCellLyrName )(int channel, char *name);

typedef int (*TFPCellLyrItem )(int channel, char *name);

typedef int (*TFPCellLyrEvalType )(int channel);

typedef int (*TFPCellLyrBlockSize )(int channel, int *bxcells, int *bycells);

typedef int (*TFPCellLyrSta )(int channel, double *dmin, double *dmax,

            double *dmean, double *dstdv);

typedef int (*TFPAccessWindowSet)(double box[4], double cellsize, double newbox[4]);

typedef int (*TFPWindowRows)(void);

typedef int (*TFPWindowCols)(void);

typedef int (*TFPWindowBox)(double box[4]);

typedef int (*TFPXY2WindowColRow)(double x, double y, int *col, int *row);

typedef int (*TFPAccessWindowClear)(void);

typedef int (*TFPPrivateAccessWindowSet)(int channel, double box[4],

                   double cellsize, double newbox[4]);

typedef int (*TFPPrivateWindowCols)(int i);

typedef int (*TFPPrivateWindowRows)(int i);

typedef int (*TFPPrivateWindowBox)(int i, double box[4]);

typedef int (*TFPXY2PrivateWindowColRow)(int i, double x, double y, int *col, int *row);

typedef int (*TFPPrivateAccessWindowClear)(int channel);

typedef char *(*TFPCAllocate1)(int number, int size);

typedef char **(*TFPCAllocate2)(int nrows, int ncols, int size);

typedef char ***(*TFPCAllocate3)(int nlyrs, int nrows, int ncols, int size);

typedef void (*TFPCFree1)(char *ptr);

typedef void (*TFPCFree2)(char **ptr, int nrows);

typedef void (*TFPCFree3)(char ***ptr, int nlyrs, int nrows);

typedef int (*TFPGetWindowRow)(int channel, int nreswinrow, CELLTYPE *rowbuf);

typedef int (*TFPGetWindowRowFloat)(int channel, int nreswinrow, float *rowbuf);

typedef int (*TFPGetWindowRowInt)(int channel, int nreswinrow, int *rowbuf);

typedef int (*TFPPutWindowRow)(int channel, int nwinrow, CELLTYPE *rowbuf);

typedef int (*TFPGetWindowBand)(int channel, int startrow, int nrows, CELLTYPE **bandbuf);

typedef int (*TFPGetWindowBandFloat)(int channel, int startrow, int nrows, float **bandbuf);

typedef int (*TFPGetWindowBandInt)(int channel, int startrow, int nrows, int **bandbuf);

typedef int (*TFPPutWindowBand)(int channel, int startrow, int nrows, CELLTYPE **bandbuf);

typedef int (*TFPGetWindowRectangle)(int channel, int rxul, int ryul,

                 int rxcells, int rycells, CELLTYPE **rbuf);

typedef int (*TFPGetWindowRectangleInt)(int channel, int rxul, int ryul,

                              int rxcells, int rycells, int **rbuf);

typedef int (*TFPGetWindowRectangleFloat)(int channel, int rxul, int ryul,

                            int rxcells, int rycells, float **rbuf);

typedef int (*TFPPutWindowRectangle)(int channel, int rxul, int ryul,

                       int rxcells, int rycells, CELLTYPE **rbuf);

typedef int (*TFPGetWindowCell)(int channel, int rescol, int resrow, CELLTYPE *cell);

typedef int (*TFPGetWindowCellFloat)(int channel, int rescol, int resrow, float *cell);

typedef int (*TFPGetWindowCellInt)(int channel, int rescol, int resrow, int *cell);

typedef int (*TFPPutWindowCellFloat)(int channel, int col, int row, double fcell);

typedef int (*TFPPutWindowCellInt)(int channel, int col, int row, int icell);

typedef int (*TFPPutWindowCell)(int channel, int col, int row, CELLTYPE cell);

typedef int (*TFPStaGetDbl)(char *clyr_name, double *dmin, double *dmax,

                      double *dmean, double *dstdv);



typedef int (*TFPStaGetMinmaxDbl)(char *clyr_name, double *dmin, double *dmax);

typedef int (*TFPGetMissingFloat)(float *floatNull);



// ==================================================================

//  Class definition

// ==================================================================

class GridIOLibrary

{

private:

  HMODULE hLib;

  bool bLibPresent;



  //true if GridIOSetup has been called,

  //false after GridIOExit has been called.

  bool m_bInitialized;



  // loading and unloading methods

  bool LoadGridIOLibrary();

  bool UnloadGridIOLibrary();



public:

  // constructor and destructor

  GridIOLibrary();

  virtual ~GridIOLibrary();



  bool Release();

  bool Initialize();



  bool isLibPresent();



  // pointers to the DLL functions

  TFPGridIOSetup GridIOSetup;

  TFPGridIOExit GridIOExit;

  TFPGridExists GridExists;

  TFPGridCopy GridCopy;

  TFPGridRename GridRename;

  TFPGridDelete GridDelete;

  TFPGridKill GridKill;

  TFPDescribeGridDbl DescribeGridDbl;

  TFPBndCellRead BndCellRead;

  TFPCellLyrExists CellLyrExists;

  TFPCellLayerCreate CellLayerCreate;

  TFPCellLayerCreateTiled CellLayerCreateTiled;

  TFPCellLayerOpen CellLayerOpen;

  TFPCellLyrClose CellLyrClose;

  TFPCellLyrCloseNoVat CellLyrCloseNoVat;

  TFPCellLyrDelete CellLyrDelete;

  TFPCellLyrGetIO CellLyrGetIO;

  TFPCellLyrSetIO CellLyrSetIO;

  TFPCellLyrName CellLyrName;

  TFPCellLyrItem CellLyrItem;

  TFPCellLyrEvalType CellLyrEvalType;

  TFPCellLyrBlockSize CellLyrBlockSize;

  TFPCellLyrSta CellLyrSta;

  TFPAccessWindowSet AccessWindowSet;

  TFPWindowRows WindowRows;

  TFPWindowCols WindowCols;

  TFPWindowBox WindowBox;

  TFPXY2WindowColRow XY2WindowColRow;

  TFPAccessWindowClear AccessWindowClear;

  TFPPrivateAccessWindowSet PrivateAccessWindowSet;

  TFPPrivateWindowCols PrivateWindowCols;

  TFPPrivateWindowRows PrivateWindowRows;

  TFPPrivateWindowBox PrivateWindowBox;

  TFPXY2PrivateWindowColRow XY2PrivateWindowColRow;

  TFPPrivateAccessWindowClear PrivateAccessWindowClear;

  TFPCAllocate1 CAllocate1;

  TFPCAllocate2 CAllocate2;

  TFPCAllocate3 CAllocate3;

  TFPCFree1 CFree1;

  TFPCFree2 CFree2;

  TFPCFree3 CFree3;

  TFPGetWindowRow GetWindowRow;

  TFPGetWindowRowFloat GetWindowRowFloat;

  TFPGetWindowRowInt GetWindowRowInt;

  TFPPutWindowRow PutWindowRow;

  TFPGetWindowBand GetWindowBand;

  TFPGetWindowBandFloat GetWindowBandFloat;

  TFPGetWindowBandInt GetWindowBandInt;

  TFPPutWindowBand PutWindowBand;

  TFPGetWindowRectangle GetWindowRectangle;

  TFPGetWindowRectangleInt GetWindowRectangleInt;

  TFPGetWindowRectangleFloat GetWindowRectangleFloat;

  TFPPutWindowRectangle PutWindowRectangle;

  TFPGetWindowCell GetWindowCell;

  TFPGetWindowCellFloat GetWindowCellFloat ;

  TFPGetWindowCellInt GetWindowCellInt;

  TFPPutWindowCellFloat PutWindowCellFloat;

  TFPPutWindowCellInt PutWindowCellInt;

  TFPPutWindowCell PutWindowCell;

  TFPStaGetDbl StaGetDbl;

  TFPStaGetMinmaxDbl StaGetMinmaxDbl;

  TFPGetMissingFloat GetMissingFloat;

};

// ==================================================================



#endif

