/**
 *  '$RCSfile: LibraryIndexTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: berkley $'
 *     '$Date: 2005/10/19 22:24:19 $'
 * '$Revision: 1.10 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.kepler.objectmanager;



import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;

import java.util.*;
import java.io.*;

//import org.kepler.objectmanager.*;
import org.kepler.objectmanager.library.*;
import org.kepler.objectmanager.lsid.*;

/**
 * <p>Title: SearchScopeTest</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public class LibraryIndexTest extends TestCase
{
  LibraryIndex index;

  public LibraryIndexTest(String name)
  {
    super(name);
  }

  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    //note that the order of these tests DOES matter.  don't change the
    //order!
    suite.addTest(new LibraryIndexTest("initialize"));
    suite.addTest(new LibraryIndexTest("testInit"));
    suite.addTest(new LibraryIndexTest("testToString"));
    suite.addTest(new LibraryIndexTest("testAdd"));
    suite.addTest(new LibraryIndexTest("testRemove"));
    suite.addTest(new LibraryIndexTest("testGetItems"));
    suite.addTest(new LibraryIndexTest("testReadWrite"));
    suite.addTest(new LibraryIndexTest("testSearch"));
    //suite.addTest(new LibraryIndexTest("testCreateLibrary"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }

  /**
   *
   */
  public void testInit()
  {
    try
    {
      index = LibraryIndex.getInstance();
    }
    catch(Exception e)
    {
      fail("Error instantiating new LibraryIndex: " + e.getMessage());
    }
  }
  
  /**
   * test the string output
   */
  public void testToString()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      index.add(ontComp, index.getRootComponent());
      String s = index.toString();
      assertTrue(s.indexOf("<LibraryIndex>") != -1);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("error printing tree: " + e.getMessage());
    }
  }
  
  /**
   * test adding to the tree
   */
  public void testAdd()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      LibraryIndexConceptItem conceptComp1 = 
        new LibraryIndexConceptItem("concept1", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept1"));
      LibraryIndexConceptItem conceptComp2 = 
        new LibraryIndexConceptItem("concept2", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      LibraryIndexComponentItem component1 =
        new LibraryIndexComponentItem("some actor 1", 
        new KeplerLSID("urn:lsid:localhost:actor:1:1"));
      LibraryIndexComponentItem component2 =
        new LibraryIndexComponentItem("some actor 2", 
        new KeplerLSID("urn:lsid:localhost:actor:2:1"));
      LibraryIndexComponentItem component3 =
        new LibraryIndexComponentItem("some actor 3", 
        new KeplerLSID("urn:lsid:localhost:actor:3:1"));
        
      index.add(ontComp, index.getRootComponent());
      index.add(conceptComp1, ontComp);
      index.add(conceptComp2, ontComp);
      index.add(component1, conceptComp1);
      index.add(component2, conceptComp2);
      index.add(component3, conceptComp2);
      
      //System.out.println(index.toString());
      //System.out.println();
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("failed to add component: " + e.getMessage());
    }
  }
  
  public void testRemove()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      LibraryIndexConceptItem conceptComp1 = 
        new LibraryIndexConceptItem("concept1", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept1"));
      LibraryIndexConceptItem conceptComp2 = 
        new LibraryIndexConceptItem("concept2", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      LibraryIndexComponentItem component1 =
        new LibraryIndexComponentItem("some actor 1", 
        new KeplerLSID("urn:lsid:localhost:actor:1:1"));
      LibraryIndexComponentItem component2 =
        new LibraryIndexComponentItem("some actor 2", 
        new KeplerLSID("urn:lsid:localhost:actor:2:1"));
      LibraryIndexComponentItem component3 =
        new LibraryIndexComponentItem("some actor 3", 
        new KeplerLSID("urn:lsid:localhost:actor:3:1"));
        
      index.add(ontComp, index.getRootComponent());
      index.add(conceptComp1, ontComp);
      index.add(conceptComp2, ontComp);
      index.add(component1, conceptComp1);
      index.add(component2, conceptComp2);
      index.add(component3, conceptComp2);
      
      index.removeAll(new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      
      assertTrue(index.toString().indexOf("concept2") == -1);
      
      //System.out.println(index.toString());
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("failed to remove component: " + e.getMessage());
    }
  }
  
  /**
   *
   */
  public void testGetItems()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      LibraryIndexConceptItem conceptComp1 = 
        new LibraryIndexConceptItem("concept1", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept1"));
      LibraryIndexConceptItem conceptComp2 = 
        new LibraryIndexConceptItem("concept2", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      LibraryIndexComponentItem component1 =
        new LibraryIndexComponentItem("some actor 1", 
        new KeplerLSID("urn:lsid:localhost:actor:1:1"));
      LibraryIndexComponentItem component2 =
        new LibraryIndexComponentItem("some actor 2", 
        new KeplerLSID("urn:lsid:localhost:actor:2:1"));
      LibraryIndexComponentItem component3 =
        new LibraryIndexComponentItem("some actor 3", 
        new KeplerLSID("urn:lsid:localhost:actor:3:1"));
      
      index.add(ontComp, index.getRootComponent());
      index.add(conceptComp1, ontComp);
      index.add(conceptComp2, ontComp);
      index.add(component1, conceptComp1);
      index.add(component2, conceptComp2);
      index.add(component3, conceptComp2);
      
      Iterator compI = index.items();
      Iterator ontoI = index.ontologies();
      //Iterator concI = index.concepts();
      
      boolean compB = false;
      boolean ontoB = false;
      
      while(compI.hasNext())
      {
        LibraryIndexItem comp = (LibraryIndexItem)compI.next();
        assertTrue(comp.getLSID().toString().equals("urn:lsid:localhost:actor:1:1") ||
          comp.getLSID().toString().equals("urn:lsid:localhost:actor:2:1") ||
          comp.getLSID().toString().equals("urn:lsid:localhost:actor:3:1"));
        compB = true;
      }
      
      while(ontoI.hasNext())
      {
        LibraryIndexItem comp = (LibraryIndexItem)ontoI.next();
        assertTrue(comp.getLSID().toString().equals("urn:lsid:localhost:ontology:1:1"));
        ontoB = true;
      }
      
      assertTrue(compB);
      assertTrue(ontoB);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("error getting components: " + e.getMessage());
    }
  }
  
  /**
   *
   */
  public void testReadWrite()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      LibraryIndexConceptItem conceptComp1 = 
        new LibraryIndexConceptItem("concept1", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept1"));
      LibraryIndexConceptItem conceptComp2 = 
        new LibraryIndexConceptItem("concept2", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      LibraryIndexComponentItem component1 =
        new LibraryIndexComponentItem("some actor 1", 
        new KeplerLSID("urn:lsid:localhost:actor:1:1"));
      LibraryIndexComponentItem component2 =
        new LibraryIndexComponentItem("some actor 2", 
        new KeplerLSID("urn:lsid:localhost:actor:2:1"));
      LibraryIndexComponentItem component3 =
        new LibraryIndexComponentItem("some actor 3", 
        new KeplerLSID("urn:lsid:localhost:actor:3:1"));
      
      index.add(ontComp, index.getRootComponent());
      index.add(conceptComp1, ontComp);
      index.add(conceptComp2, ontComp);
      index.add(component1, conceptComp1);
      index.add(component2, conceptComp2);
      index.add(component3, conceptComp2);
      
      index.serialize(); //serialize the tree
      index = LibraryIndex.getNewInstance(); //de-serialize the tree
      
      Iterator compI = index.items();
      boolean compB = false;
      while(compI.hasNext())
      { //make sure the components are back
        LibraryIndexItem comp = (LibraryIndexItem)compI.next();
        assertTrue(comp.getLSID().toString().equals("urn:lsid:localhost:actor:1:1") ||
          comp.getLSID().toString().equals("urn:lsid:localhost:actor:2:1") ||
          comp.getLSID().toString().equals("urn:lsid:localhost:actor:3:1"));
        compB = true;
      }
      assertTrue(compB);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("could not read or write the serialized file: " + e.getMessage());
    }
  }
  
  /**
   *
   */
  public void testSearch()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      LibraryIndexConceptItem conceptComp1 = 
        new LibraryIndexConceptItem("concept1", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept1"));
      LibraryIndexConceptItem conceptComp2 = 
        new LibraryIndexConceptItem("concept2", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      LibraryIndexComponentItem component1 =
        new LibraryIndexComponentItem("some actor 1", 
        new KeplerLSID("urn:lsid:localhost:actor:1:1"));
      LibraryIndexComponentItem component2 =
        new LibraryIndexComponentItem("some actor 2", 
        new KeplerLSID("urn:lsid:localhost:actor:2:1"));
      LibraryIndexComponentItem component3 =
        new LibraryIndexComponentItem("some actor 3", 
        new KeplerLSID("urn:lsid:localhost:actor:3:1"));
      
      index.add(ontComp, index.getRootComponent());
      index.add(conceptComp1, ontComp);
      index.add(conceptComp2, ontComp);
      index.add(component1, conceptComp1);
      index.add(component2, conceptComp2);
      index.add(component3, conceptComp2);
            
      assertTrue(index.isIndexed(new KeplerLSID("urn:lsid:localhost:ontology:1:1")));
      assertTrue(index.isIndexed(new KeplerLSID("urn:lsid:localhost:actor:1:1")));
      assertTrue(index.isIndexed(new KeplerLSID("urn:lsid:localhost:actor:2:1")));
      assertTrue(index.isIndexed(new KeplerLSID("urn:lsid:localhost:actor:3:1")));
      assertTrue(index.isIndexed(new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2")));
      assertTrue(!index.isIndexed(new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept44")));
      assertTrue(!index.isIndexed(new KeplerLSID("urn:lsid:localhost:actor:5:1")));
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("failed to find lsid: " + e.getMessage());
    }
  }
  
  /**
   *
   */
  public void testCreateLibrary()
  {
    try
    {
      index = LibraryIndex.getInstance();
      index.clear();
      LSIDTree.getInstance().clearTree();
      LibraryIndexOntologyItem ontComp = 
        new LibraryIndexOntologyItem("newOnto", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1"));
      LibraryIndexConceptItem conceptComp1 = 
        new LibraryIndexConceptItem("concept1", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept1"));
      LibraryIndexConceptItem conceptComp2 = 
        new LibraryIndexConceptItem("concept2", 
        new KeplerLSID("urn:lsid:localhost:ontology:1:1#concept2"));
      LibraryIndexComponentItem component1 =
        new LibraryIndexComponentItem("some actor 1", 
        new KeplerLSID("urn:lsid:localhost:actor:1:1"));
      LibraryIndexComponentItem component2 =
        new LibraryIndexComponentItem("some actor 2", 
        new KeplerLSID("urn:lsid:localhost:actor:2:1"));
      LibraryIndexComponentItem component3 =
        new LibraryIndexComponentItem("some actor 3", 
        new KeplerLSID("urn:lsid:localhost:actor:3:1"));
      
      index.add(ontComp, index.getRootComponent());
      index.add(conceptComp1, ontComp);
      index.add(conceptComp2, ontComp);
      index.add(component1, conceptComp1);
      index.add(component2, conceptComp2);
      index.add(component3, conceptComp2);
      
      ptolemy.vergil.tree.EntityTreeModel model = index.createLibrary();
    }
    catch(Exception e)
    {
      fail("failed to create library");
    }
  }
}