/**
 *    '$RCSfile: GarpPresampleLayers.java,v $'
 *
 *     '$Author: ruland $'
 *       '$Date: 2005/11/01 20:39:06 $'
 *   '$Revision: 1.12 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.garp;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;


/*<p></p>*/

public class GarpPresampleLayers extends TypedAtomicActor
{
  // FileParameters
  public FileParameter layersetFilenameParameter =
                      new FileParameter(this, "layersetFilenameParameter");
  public FileParameter dataPointFileNameParameter =
                      new FileParameter(this, "dataPointFileNameParameter");
  public FileParameter cellSetFileNameParameter =
                      new FileParameter(this, "cellSetFileNameParameter");
  //input ports
  public TypedIOPort layersetFilename = new TypedIOPort(this,
    "layersetFilename", true, false);
  public TypedIOPort dataPointFileName = new TypedIOPort(this,
    "dataPointFileName", true, false);
  public TypedIOPort cellSetFileName = new TypedIOPort(this, "cellSetFileName",
    true, false);
  //output ports
  public TypedIOPort cellSetFileNameOutput = new TypedIOPort(this, "cellSetFileNameOutput",
    false, true);

  /**
   * Garp presample layer actor
   */
  public GarpPresampleLayers(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException
  {
    super(container, name);
    layersetFilename.setTypeEquals(BaseType.STRING);
    dataPointFileName.setTypeEquals(BaseType.STRING);
    cellSetFileName.setTypeEquals(BaseType.STRING);
    cellSetFileNameOutput.setTypeEquals(BaseType.STRING);

     _attachText("_iconDescription",
     "<svg>\n"
     + "<rect x=\"0\" y=\"0\" "
     + "width=\"66\" height=\"42\" "
     + "style=\"fill:white\"/>\n"
     + "<text x=\"12\" y=\"16\" "
     + "style=\"font-size:14; fill:blue; font-family:SansSerif\">"
     + "GARP</text>\n"
     + "<text x=\"4\" y=\"34\" "
     + "style=\"font-size:12; fill:blue; font-family:SansSerif\">"
     + "Presample</text>\n"
     + "</svg>\n");

  }

  /**
   *
   */
  public void initialize()
    throws IllegalActionException
  {
  }

  /**
   *
   */
  public boolean prefire()
    throws IllegalActionException
  {
    return super.prefire();
  }

  /**
   *
   */
  public void fire()
    throws IllegalActionException
  {
    System.out.println("Firing GarpPresampleLayers");
    super.fire();
    String layersetFilenameStr = "";
    String dataPointFileNameStr = "";
    String cellSetFileNameStr = "";
    if (layersetFilename.numberOfSources() > 0) {
      if (!layersetFilename.hasToken(0)) return;
      StringToken layerSetFileNameToken = (StringToken)layersetFilename.get(0);
      layersetFilenameStr = layerSetFileNameToken.stringValue();
      layersetFilenameParameter.setExpression(layersetFilenameStr);
    } else {
      layersetFilenameStr = layersetFilenameParameter.asFile().getPath();
    }

    if (dataPointFileName.numberOfSources() > 0) {
      if (!dataPointFileName.hasToken(0)) return;
      StringToken dataPointFileNameToken = (StringToken)dataPointFileName.get(0);
      dataPointFileNameStr = dataPointFileNameToken.stringValue();
      dataPointFileNameParameter.setExpression(dataPointFileNameStr);
    } else {
      dataPointFileNameStr = dataPointFileNameParameter.asFile().getPath();
    }

    if (cellSetFileName.numberOfSources() > 0) {
      if (!cellSetFileName.hasToken(0)) return;
      StringToken cellSetFileNameToken = (StringToken)cellSetFileName.get(0);
      cellSetFileNameStr = cellSetFileNameToken.stringValue();
      cellSetFileNameParameter.setExpression(cellSetFileNameStr);
    } else {
      cellSetFileNameStr = cellSetFileNameParameter.asFile().getPath();
    }

    System.out.println("Starting GarpPresampleLayers JNI Code");
    new GarpJniGlue().PresampleLayers(layersetFilenameStr,
      dataPointFileNameStr, cellSetFileNameStr);

    cellSetFileNameOutput.broadcast(new StringToken(cellSetFileNameStr));

    System.out.println("Finished with GarpPresampleLayers JNI Code");
  }
}
